@@ -13,7 +13,7 @@ from xraydb import (chemparse, material_mu, material_mu_components,
                     atomic_symbol, atomic_mass, atomic_density, xray_edges,
                     xray_edge, xray_lines, xray_line, fluor_yield,
                     ck_probability, core_width, guess_edge,
-                    xray_delta_beta, mirror_reflectivity,
+                    xray_delta_beta, darwin_width, mirror_reflectivity,
                     ionchamber_fluxes, XrayDB)
 
 
@@ -209,14 +209,23 @@ def test_elam_coh_incoh():
 def test_mu_chantler():
     en = np.linspace(13000, 17000, 51)
 
-    mu_total = np.array([64.44, 141.67, 142.46, 143.20, 143.97, 144.66, 144.12,
-                         142.86, 140.56, 138.30, 136.10, 133.99, 131.92, 129.85,
-                         127.82, 125.84, 123.90, 122.01, 120.21, 118.44, 116.67,
-                         114.91, 113.16, 111.48, 109.87, 108.29, 106.74, 105.16,
-                         144.38, 142.63, 141.02, 139.34, 137.71, 136.15, 134.59,
-                         133.04, 149.40, 147.61, 145.77, 143.97, 142.19, 140.45,
-                         138.75, 137.07, 135.43, 133.82, 132.24, 130.69, 129.17,
-                         127.64, 126.13])
+    mu_total = np.array([ 67.24736219, 156.95328746, 155.78747252,
+                          153.64333955, 151.35899169, 149.0524903 ,
+                          146.74603265, 144.48912501, 142.28067496,
+                          140.09062866, 137.93546991, 135.8273229 ,
+                          133.76088713, 131.73138628, 129.74341174,
+                          127.79595645, 125.88827932, 124.01945648,
+                          122.18963972, 120.39733297, 118.64336375,
+                          116.92801058, 115.25227237, 113.61866015,
+                          112.02400421, 110.52333492, 109.0762108 ,
+                          108.31373956, 146.25741077, 144.9280078 ,
+                          143.11197774, 141.21399554, 139.35518911,
+                          137.5608917 , 135.82234118, 134.4535719 ,
+                          148.93741603, 149.77263523, 148.14548539,
+                          146.37740382, 144.62801998, 142.87203507,
+                          141.14422981, 139.43965338, 137.76256418,
+                          136.10754019, 134.47907574, 132.8679473 ,
+                          131.28344089, 129.71407476, 128.1707394 ])
 
     mu_photo = np.array([63.33, 153.06, 151.92, 149.80, 147.54, 145.26, 142.98,
                          140.74, 138.56, 136.39, 134.26, 132.17, 130.13, 128.12,
@@ -244,7 +253,7 @@ def test_mu_chantler():
     assert_allclose(mu_total, chantler_data('Pb', en, 'mu_total'), rtol=0.01)
     assert_allclose(mu_total, chantler_data('Pb', en, 'mu'), rtol=0.01)
 
-    assert_allclose(64.44, chantler_data('Pb', 13000, 'mu'), rtol=0.01)
+    assert_allclose(67.25, chantler_data('Pb', 13000, 'mu'), rtol=0.01)
 
 
 def test_f1f2_chantler():
@@ -271,6 +280,11 @@ def test_f1f2_chantler():
     assert_allclose(f1, chantler_data('Au', en, 'f1'), rtol=0.01)
     assert_allclose(f2, chantler_data('Au', en, 'f2'), rtol=0.01)
 
+def test_f1f2_chantler_mo():
+    en = np.linspace(1000, 500000, 501)
+    f1 = f1_chantler('Mo', en)
+    assert(len(f1) > 400)
+
 def test_chantler_energies():
 
     en = np.array([6876.64, 6945.41, 7014.86, 7085.01, 7155.86, 7227.42,
@@ -524,19 +538,20 @@ def test_delta_beta():
                         1.84314908e-06, 1.82363251e-06, 1.80437431e-06,
                         1.78600492e-06, 1.76816730e-06, 1.75055477e-06])
 
-    atten = np.array([0.00365468, 0.00368604, 0.00371757, 0.00374933,
-                      0.00378128, 0.0038134 , 0.0038457 , 0.00387818,
-                      0.00391084, 0.00394364, 0.00397654, 0.00400961,
-                      0.00404286, 0.0040764 , 0.00411013, 0.00414403,
-                      0.00417797, 0.00421199, 0.00424618, 0.0042806 ,
-                      0.00431548, 0.00435053, 0.00438577, 0.00442101,
-                      0.00445637, 0.00449191, 0.00452767, 0.00456372,
-                      0.00459996, 0.00463653, 0.00467309, 0.00065701,
-                      0.00066161, 0.00066625, 0.00067084, 0.00067545,
-                      0.00068008, 0.00068474, 0.00068943, 0.00069413,
-                      0.00069886, 0.00070361, 0.00070839, 0.00071318,
-                      0.000718, 0.00072285, 0.00072772, 0.0007326,
-                      0.00073751, 0.00074245, 0.0007474 ])
+
+    atten = np.array([0.00376372, 0.00379486, 0.00382616, 0.00385755,
+                      0.00388909, 0.00392079, 0.00395256, 0.00398442,
+                      0.00401644, 0.00404858, 0.00408074, 0.00411306,
+                      0.00414555, 0.00417787, 0.00421033, 0.00424294,
+                      0.0042754 , 0.00430777, 0.00434029, 0.00437275,
+                      0.00440432, 0.00443603, 0.00446788, 0.00449701,
+                      0.00452518, 0.00455344, 0.00457579, 0.00458077,
+                      0.00458572, 0.0045724 , 0.00431164, 0.00065601,
+                      0.00065126, 0.00065474, 0.00065973, 0.00066474,
+                      0.00066978, 0.00067505, 0.00068046, 0.0006859 ,
+                      0.00069138, 0.00069693, 0.00070251, 0.00070812,
+                      0.00071377, 0.00071946, 0.00072517, 0.00073092,
+                      0.00073644, 0.00074186, 0.0007473 ])
 
     en = np.linspace(6500, 7500, 51)
     d, b, a = xray_delta_beta('Fe2O3', 5.25, en)
@@ -553,11 +568,43 @@ def test_mirror_reflectivity():
                     0.9607325 , 0.95705164, 0.94578432, 0.69383757, 0.65937538,
                     0.57222587, 0.40829232, 0.26919575, 0.18904094, 0.14024917])
     r1 = mirror_reflectivity('Rh', 0.0025, np.arange(1000, 31000, 1000))
-    assert_allclose(rh1, r1, rtol=0.005)    
+    assert_allclose(rh1, r1, rtol=0.005)
 
     assert_allclose(mirror_reflectivity('Pt', 0.001, 80000), 0.74, rtol=0.005)
 
-    
+def test_darwin_width():
+    def dw_dide(energy, crystal, hkl):
+        """estimate darwin width in energy from min/max derivative"""
+        dat = darwin_width(energy, crystal=crystal, hkl=hkl)
+        dide = np.gradient(dat.intensity) / np.gradient(dat.denergy)
+        return (dat.denergy[np.where(dide==dide.min())] -
+                dat.denergy[np.where(dide==dide.max())])[0]
+
+    def dw_fwhm(energy, crystal, hkl):
+        """as-calculated darwin width in energy (eV)"""
+        return darwin_width(energy, crystal=crystal, hkl=hkl).energy_fwhm
+
+    # fwhm from derivatives:
+    assert_allclose(dw_dide( 5000, 'Si', (1, 1, 1)), 0.685, rtol=0.01)
+    assert_allclose(dw_dide(10000, 'Si', (1, 1, 1)), 1.340, rtol=0.01)
+
+    assert_allclose(dw_dide(10000, 'Si', (2, 2, 0)), 0.595, rtol=0.01)
+    assert_allclose(dw_dide(15000, 'Si', (2, 2, 0)), 0.891, rtol=0.01)
+
+    assert_allclose(dw_dide(10000, 'Si', (3, 1, 1)), 0.288, rtol=0.01)
+    assert_allclose(dw_dide(20000, 'Si', (3, 1, 1)), 0.565, rtol=0.01)
+
+    # fwhm as cacluated
+    assert_allclose(dw_fwhm( 5000, 'Si', (1, 1, 1)), 0.723, rtol=0.01)
+    assert_allclose(dw_fwhm(10000, 'Si', (1, 1, 1)), 1.418, rtol=0.01)
+
+    assert_allclose(dw_fwhm(10000, 'Si', (2, 2, 0)), 0.624, rtol=0.01)
+    assert_allclose(dw_fwhm(15000, 'Si', (2, 2, 0)), 0.927, rtol=0.01)
+
+    assert_allclose(dw_fwhm(10000, 'Si', (3, 1, 1)), 0.301, rtol=0.01)
+    assert_allclose(dw_fwhm(20000, 'Si', (3, 1, 1)), 0.593, rtol=0.01)
+
+
 def test_ionchamber_fluxes():
         ic1 = ionchamber_fluxes(gas='helium', volts=1.25, length=200.0,
                               energy=10000.0, sensitivity=1.e-9)
@@ -570,7 +617,7 @@ def test_ionchamber_fluxes():
 
         assert_allclose(ic2.photo, 13575282.2, rtol=0.01)
         assert_allclose(ic2.incident, 23102328.0, rtol=0.01)
-        
+
         ic3 = ionchamber_fluxes(gas={'nitrogen':0.5, 'helium': 0.5}, volts=1.25,
                                 length=200.0, energy=10000.0, sensitivity=1.e-9)
 
