@@ -5,6 +5,7 @@ import numpy as np
 from .utils import (R_ELECTRON_CM, AVOGADRO, PLANCK_HC,
                     QCHARGE, SI_PREFIXES, index_nearest)
 
+R0 = 1.e8 * R_ELECTRON_CM 
 
 from .xraydb import XrayDB,  XrayLine
 from .chemparser import chemparse
@@ -13,6 +14,12 @@ fluxes = namedtuple('IonChamberFluxes', ('photo',
                                          'incident',
                                          'transmitted'))
 
+DarwinWidth = namedtuple('DarwinWidth', ('theta', 'theta_offset',
+                                         'theta_width', 'theta_fwhm',
+                                         'energy_width', 'energy_fwhm',
+                                         'zeta', 'dtheta', 'denergy',
+                                         'intensity'))
+
 
 _edge_energies = {'k': np.array([-1.0, 13.6, 24.6, 54.7, 111.5, 188.0,
                                  284.2, 409.9, 543.1, 696.7, 870.2, 1070.8,
@@ -589,11 +596,12 @@ def core_width(element, edge=None):
     return xdb.corehole_width(element, edge=edge)
 
 def ionization_potential(gas):
-    """return effective ionization potential for a gas, as appropriate for
-    ionization chambers in the linear regime (not 'proportional counter' regime)
+    """return effective ionization potential for a gas or diode semiconductor, as
+    appropriate for ionization chambers in the linear regime (not in the
+    'proportional counter' regime) or for PIN photodiodes (not in 'avalanche' mode).
 
     Args:
-        gas (string):  name of gas
+        gas (string):  name of gas or 'Si' or 'Ge'
 
     Returns:
         ionization potential in eV
@@ -607,7 +615,7 @@ def ionization_potential(gas):
            -------------------- ----------------
             hydrogen, H           36.5
             helium, He            41.3
-            nitrogren, N, N2      34.8
+            nitrogen, N, N2       34.8
             oxygen, O, O2         30.8
             neon, Ne              35.4
             argon, Ar             26.4
@@ -616,6 +624,8 @@ def ionization_potential(gas):
             air                   33.8
             methane, CH4          27.3
             carbondioxide, CO2    33.0
+            silicon, Si            3.68
+            germanium, Ge          2.97
            ==================== ================
 
        If the gas is not recognized the default value of 32.0 eV will be returned.
@@ -798,9 +808,10 @@ def mirror_reflectivity(formula, theta, energy, density=None,
 def ionchamber_fluxes(gas='nitrogen', volts=1.0, length=100.0,
                       energy=10000.0, sensitivity=1.e-6,
                       sensitivity_units='A/V'):
-    
-    """return ion chamber fluxes for a gas or mixture of gases, ion chamber
-    length, X-ray energy, recorded voltage and current amplifier sensitivity.
+
+    """return ion chamber and PIN diode fluxes for a gas, mixture of gases, or
+    semiconductor material, ion chamber length (or diode thickness), X-ray energy,
+    recorded voltage and current amplifier sensitivity.
 
     Args:
         gas (string or dict):  name or formula of fill gas (see note 1) ['nitrogen']
@@ -815,52 +826,52 @@ def ionchamber_fluxes(gas='nitrogen', volts=1.0, length=100.0,
         named tuple IonchamberFluxes with fields
 
             `photo`       flux absorbed by photo-electric effect in Hz,
-            
+
             `incident`    flux of beam incident on ion chamber in Hz,
-            
-            `transmitted` flux of beam output of ion chamber in Hz 
+
+            `transmitted` flux of beam output of ion chamber in Hz
 
     Notes:
-       1. The gas argument can either be a string for the name of chemical
-          formula for the gas, or dictionary with keys that are gas names or
-          formulas and values that are the relative fraction for mixes gases.
+       1. The gas value can either be a string for the name of chemical
+          formula for the gas or diode material, or dictionary with keys
+          that are gas names or formulas and values that are the relative
+          fraction for mixes gases.  For diode materials, mixtures are not
+          supported.
 
           The gas formula is used in two ways:
              a) to get the photo- and total- absorption coefficient, and
-             b) to get the effective ionization potential for the gas.
+             b) to get the effective ionization potential for the material.
 
           The effective ionization potentials are known for a handful of
-          gases (see `ionization_potential` function), all ranging between
-          20 and 45 eV. For unknown gases the value of 32.0 eV will be
-          used.
+          gases and diodes (see `ionization_potential` function), and range
+          between 20 and 45 eV for gases, and around 3 eV for semiconductors.
+          For unknown gases the value of 32.0 eV will be used.
 
        2. The `sensitivity` and `sensitivity_units` arguments have some overlap
           to specify the sensitivity or gain of the current amplifier.
           Generally, the units in `A/V`, but you can add a common SI prefix of
- 
-          'p', 'pico', 'n', 'nano', '\u03bc', 'u', 'micro', 'm', 'milli'
-          
+
+          'p', 'pico', 'n', 'nano', (unicode 'u03bc'), 'u', 'micro', 'm', 'milli'
+
           so that, for example
              ionchamber_fluxes(..., sensitivity=1.e-6)
-          and 
+          and
              ionchamber_fluxes(..., sensitivity=1, sensitivity_units='uA/V')
 
-          will both give a sensitivity of 1 microAmp / Volt . 
-               
+          will both give a sensitivity of 1 microAmp / Volt .
+
     Examples:
         >>> ionchamber_fluxes(gas='helium', volts=1.25, length=200.0,
                               energy=10000.0, sensitivity=1.e-9)
         IonChamberFluxes(photo=16110895.3, incident=15452608024.6, transmitted=15316549138.8)
-
         >>> ionchamber_fluxes(gas='nitrogen', volts=1.25, length=200.0,
                               energy=10000.0, sensitivity=1.e-9)
         IonChamberFluxes(photo=13575282.2, incident=23102328.0, transmitted=8759458.7)
-
         >>> ionchamber_fluxes(gas={'nitrogen':0.5, 'helium': 0.5}, volts=1.25,
                               length=200.0, energy=10000.0, sensitivity=1.e-9)
         IonChamberFluxes(photo=14843088.8, incident=7737855176.4, transmitted=7662654298.8)
 
-    
+
     """
     from .materials import material_mu
 
@@ -868,13 +879,13 @@ def ionchamber_fluxes(gas='nitrogen', volts=1.0, length=100.0,
 
     units = sensitivity_units.replace('Volts', 'V').replace('Volt', 'V')
     units = units.replace('Amperes', 'A').replace('Ampere', 'A')
-    units = units.replace('Amps', 'A').replace('Amp', 'A')    
+    units = units.replace('Amps', 'A').replace('Amp', 'A')
     units = units.replace('A/V', '')
     sensitivity *= SI_PREFIXES.get(units, 1)
 
     if isinstance(gas, str):
         gas = {gas: 1.0}
-    
+
     gas_total = 0.0
     gas_comps = []
     for gname, frac in gas.items():
@@ -901,7 +912,140 @@ def ionchamber_fluxes(gas='nitrogen', volts=1.0, length=100.0,
         flux_out   = flux_in * np.exp(-length*mu_total)
 
         fphoto += flux_photo
-        fin    += flux_in 
+        fin    += flux_in
         fout   += flux_out
 
     return fluxes(photo=fphoto, incident=fin,transmitted=fout)
+
+
+def darwin_width(energy, crystal='Si', hkl=(1, 1, 1), a=None,
+                 polarization='s', ignore_f2=False, ignore_f1=False, m=1):
+    
+    """darwin width for a crystal reflection and energy
+
+    Args:
+      energy (float):    X-ray energy in eV
+      crystal (string):  name of crystal (one of 'Si', 'Ge', or 'C') ['Si']
+      hkl (tuple):       h, k, l for reflection  [(1, 1, 1)]
+      a (float or None): lattice constant [None - use built-in value]
+      polarization ('s','p'): mono orientation relative to X-ray polarization ['s']
+      ignore_f1 (bool):  ignore contribution from f1 - dispersion (False)
+      ignore_f2 (bool):  ignore contribution from f2 - absorption (False)
+      m (int):           order of reflection    [1]
+      
+    Returns:
+
+      A named tuple 'DarwinWidth' with the following fields
+
+        `theta`        float, nominal Bragg angle, in rad,
+
+        `theta_offset` float, angular offset from Bragg angle, in rad,
+
+        `theta_width`  float, estimated angular Darwin width, in rad,
+
+        `theta_fwhm`   float, estimated FWHM of angular intensity, in rad,
+
+        `energy_width` float, estimated angular Darwin width, in rad,
+
+        `energy_fwhm`  float, estimated FWHM of energy intensity, in eV,
+
+        `zeta`         nd-array of Zeta parameter (delta_Lambda / Lambda),
+
+        `dtheta`       nd-array of angles away from Bragg angle, theta in rad,
+
+        `denergy`      nd-array of energies away from Bragg energy, in eV,
+
+        `intensity`    nd-array of reflected intensity
+
+    Notes:
+
+     1. This follows the calculation from section 6.4 of
+        Elements of Modern X-ray Physics, 2nd Edition
+        J Als-Nielsen, and D. McMorrow.
+
+     2. Only diamond structures (Si, Ge, diamond) and sigma
+        polarization are currently supported.
+
+     3. Default values of lattice constant `a` are in Angstroms:
+        5.4309 for Si, 5.6578, for 'Ge', and 3.567 for 'C'.
+        
+     3. The `theta_width` and `energy_width` values will closely match the
+        width of the intensity profile that would = 1 when ignoring the
+        effect of absorption.  These are the values commonly reported as
+        'Darwin Width'.  The value reported for `theta_fwhm' and
+        `energy_fwhm` are larger than this by sqrt(9/8) ~= 1.06.
+        
+     4. Polarization of 's' would be for a vertically deflecting crystal and
+        a horizontally-polarized, as for most synchrotron beamlines.
+        
+    Examples:
+        >>> dw = darwin_width(10000, crystal='Si', hkl=(1, 1, 1))
+        >>> dw.theta_width, dw.energy_width
+        (2.8593683930207114e-05, 1.4177346002236872)
+
+    """
+    lattice_constants = {'Si': 5.4309, 'Ge': 5.6578, 'C': 3.567}
+
+    h_, k_, l_ = hkl
+    hklsum = (h_ + k_ + l_)
+    if hklsum % 4 == 0 and (h_ % 2 == 0 and k_ % 2 == 0 and l_ % 2 == 0):
+        eqr = 8
+    elif (h_ % 2 == 1 and k_ % 2 == 1 and l_ % 2 == 1): # all odd
+        eqr =4*np.sqrt(2)  
+    else:
+        raise ValueError("hkl must sum to 4 or be all odd")
+
+    if a is None:
+        a = lattice_constants[crystal.title()]
+    dspace = a / np.sqrt(h_*h_ + k_*k_ + l_*l_)
+    lambd  = PLANCK_HC / energy
+    theta  = np.arcsin(lambd/(2*dspace))
+
+    q  = 0.5 / dspace
+    f1 = f2 = 0
+    if not ignore_f1: 
+        f1 = f1_chantler(crystal, energy)
+    if not ignore_f2:         
+        f2 = f2_chantler(crystal, energy)
+
+    gscale = 2 * (dspace)**2 * R0 / (m*a**3)
+    if polarization.startswith('p'):
+        gscale *= np.cos(2*theta)
+        
+    g0 = gscale * 8   * (f0(crystal, 0)[0] + f1 - 1j*f2)
+    g  = gscale * eqr * (f0(crystal, q)[0] + f1 - 1j*f2)
+
+    total = abs(2*g/(m*np.pi))
+    fwhm  = total * 3/(2*np.sqrt(2))  # where do A-N&M get this factor?
+
+    zeta_offset = g0.real/np.pi
+    theta_offset = np.tan(theta)*zeta_offset
+
+    # as a check, the following formula from L Berman (and X0h doc)
+    # will give identical results as theta_width. [sin(2x)= 2sin(x)*cos(x)]
+    # dw_lb = 2*R0*lambd**2 * eqr*abs(f0(crystal, q)[0] + f1 - 1j*f2)/(m*np.pi*a**3* np.sin(2*theta))
+    
+    #  hueristic zeta range and step sizes for crystals:
+    sz =  zeta_offset
+
+    zeta = np.concatenate((np.arange(-1.5*sz,    0, 0.05*total),
+                           np.arange(0,       2*sz, 0.01*total),
+                           np.arange(2*sz,  3.5*sz, 0.05*total)))
+    xc = (m*np.pi*zeta - g0)/g
+    _p = np.where(xc.real > 1)[0]
+    _n = np.where(xc.real < -1)[0]
+
+    r     = (xc - 1j * np.sqrt(1 - xc**2))
+    r[_p] = (xc - np.sqrt(xc**2 -1))[_p]
+    r[_n] = (xc + np.sqrt(xc**2 -1))[_n]
+
+    return DarwinWidth(theta=theta,
+                       theta_offset=theta_offset, 
+                       theta_width=total*np.tan(theta),
+                       theta_fwhm=fwhm*np.tan(theta),
+                       energy_width=total*energy,
+                       energy_fwhm=fwhm*energy, 
+                       zeta=zeta,                       
+                       dtheta=zeta*np.tan(theta),
+                       denergy=-zeta*energy,
+                       intensity=abs(r*r.conjugate()))
