@@ -28,10 +28,9 @@ __version__ = '1.4'
 def make_engine(dbname):
     "create engine for sqlite connection"
     return create_engine('sqlite:///%s' % (dbname),
-                         # poolclass=SingletonThreadPool,
+                         poolclass=SingletonThreadPool,
                          connect_args={'check_same_thread': False})
 
-
 def isxrayDB(dbname):
     """whether a file is a valid XrayDB database
 
@@ -242,6 +241,7 @@ class XrayDB():
         if column == 'mu':
             column = 'mu_total'
         ty = np.array(json.loads(getattr(row, column)))[nemin:nemax+1]
+        ty[np.where(abs(ty) < 1.e-99)] =  1.e-99
         if column == 'f1':
             out = UnivariateSpline(te, ty, s=smoothing)(energy)
         else:
@@ -708,7 +708,7 @@ class XrayDB():
         return self.cross_section_elam(element, energies, kind='coh')
 
     def incoherent_cross_section_elam(self, element, energies):
-        """returns incoherenet scattering cross section for an element
+        """return incoherenet scattering cross section for an element
         at energies (in eV)
 
         returns values in units of cm^2 / gr
@@ -721,3 +721,24 @@ class XrayDB():
         Data from Elam, Ravel, and Sieber.
         """
         return self.cross_section_elam(element, energies, kind='incoh')
+
+    def ionization_potential(self, gas):
+        """return effective ionization potential for a gas,
+        as appropriate for ionization chambers in the linear
+        regime (not in the 'proportional counter' regime)
+
+        returns values in units of eV
+
+        argument
+        ---------
+        gas (string):  name of gas
+
+        notes
+        -----
+        Data from G. F. Knoll, Radiation Detection and Measurement, Table 5-1.
+        """
+        itab = self.tables['ionization_potentials']
+        out = self.query(itab).filter(itab.c.gas == gas).all()
+        if len(out) != 1:
+            raise ValueError('unknown gas for ionization potential: %s' % gas)
+        return float(out[0].potential)
